"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateDataSource = void 0;
const tslib_1 = require("tslib");
const schematics_open_api_1 = require("@rxap/schematics-open-api");
const path_1 = require("path");
const ts_morph_1 = require("ts-morph");
const const_1 = require("./const");
const core_1 = require("@angular-devkit/core");
const { dasherize, classify } = core_1.strings;
function GenerateDataSource(parameter) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parameter.method.toUpperCase() !== 'GET') {
            return;
        }
        const operationId = parameter.operationId;
        const name = [operationId, const_1.DATA_SOURCE_FILE_SUFFIX].join('.');
        const fileName = (0, path_1.join)(const_1.DATA_SOURCE_BASE_PATH, dasherize(name) + '.ts');
        const sourceFile = parameter.project.createSourceFile(fileName);
        const importStructures = [
            {
                moduleSpecifier: '@rxap/open-api/data-source',
                namedImports: [
                    { name: 'RxapOpenApiDataSource' },
                    { name: 'OpenApiDataSource' }
                ]
            },
            {
                moduleSpecifier: '@angular/core',
                namedImports: [{ name: 'Injectable' }]
            }
        ];
        const responseType = (0, schematics_open_api_1.GetResponseType)(parameter);
        const parameterType = (0, schematics_open_api_1.GetParameterType)(parameter);
        if (!['void', 'any'].includes(responseType)) {
            importStructures.push({
                moduleSpecifier: `../responses/${dasherize(responseType.replace(/Response$/, ''))}.response`,
                namedImports: [{ name: responseType }]
            });
        }
        if (!['void', 'any'].includes(parameterType)) {
            importStructures.push({
                moduleSpecifier: `../parameters/${dasherize(parameterType.replace(/Parameter$/, ''))}.parameter`,
                namedImports: [{ name: parameterType }]
            });
        }
        const classStructure = {
            name: classify(name.replace(/\./g, '-')),
            decorators: [
                {
                    name: 'Injectable',
                    arguments: ts_morph_1.Writers.object({
                        providedIn: (writer) => writer.quote('root')
                    })
                },
                {
                    name: 'RxapOpenApiDataSource',
                    arguments: (writer) => writer.quote(operationId)
                }
            ],
            extends: (writer) => {
                writer.write('OpenApiDataSource');
                writer.write('<');
                writer.write(responseType);
                writer.write(', ');
                writer.write(parameterType);
                writer.write('>');
            },
            isExported: true
        };
        sourceFile.addImportDeclarations(importStructures);
        sourceFile.addClass(classStructure);
    });
}
exports.GenerateDataSource = GenerateDataSource;
//# sourceMappingURL=generate-data-source.js.map